"""
This module is used for creating and showing menus.
"""
import curses

from menu_zerobeef.data import (ItemMultiSelection, ItemSingleSelection,
                                MenuItem)


class ApplicationMenu():
    """
    This is the class for the menu.
    The class handles both drawing
    and capturing keystrokes. If you
    want to change the functionality
    it is suggested that you extend
    and override the specific functions
    from this class that you want to change.

    Functions
    ---------
    __init__ : self, stdscr, args
        Initiailizes the instance of the class
        with the information given, it requires
        items and an standard screen from curses.
        The standard screen from curses can be
        obtained from curses.wrapper or similar.
    show_menu : self
        Prints the menu on the screen for the
        end user and stops to capture a keystroke
        which will determine the action to take.
    actions : self
        Reads the keystroke from the end user
        and either moves up or down one position
        in the menu. It can also execute any action
        that the menu item has, mark the item as
        selected or return a list or a single item.
    exit_menu : self
        This changes one variable that determines
        whether the loop that presents the menu
        should be on or off. The variable will
        be set to an off state using this function,
        which means that the menu will exit.
    remark_items : self
        It goes through all menu items and sends
        the current position to all of them. Each
        item will then check wether it is located
        at that position or not. Each item will
        also check if it's already active or not.
        If it has the position number that matches
        or is active then it will change state.

    Parameters
    ----------
    stdscr : Any
        An object which lets the class
        draw items on screen.
    args : List/Tuple
        A tuple or a list of objects
        that would otherwise make the
        argument list too long.
    
    Returns
    -------
    An initialized instance of the class
    ApplicationMenu.
    """

    def __init__(self, stdscr, args=()):
        """
        Initializes a menu for different usecases.

        This function initializes the settings for
        a menu with different settings. Settings
        such as multiselection, singleselection or
        just a plain menu.

        Parameters
        ----------
        self : class
            A pointer to the class itself to be able
            to access all the neccessary items set
            in this function.
        """
        self.stdscr = stdscr
        self.cursor_position = 0
        self.items = args[0]
        self.menu_title = args[1]
        self.loop = True
        self.init_scr_x = args[2] if len(args) > 2 else 1
        self.init_scr_y = args[3] if len(args) > 3 else 2
        curses.init_pair(1, curses.COLOR_BLACK, -1)

    def show_menu(self):
        """
        Presents the menu to the end user.

        The function clears the screen and
        writes the title as well as all the
        menu items that it has from the time
        of being created. It uses str function
        on each item as that is coded as __str__
        which returns a specific string for
        each type of menu item. It also looks
        at the style for each menu item, whether
        the item is selected or not and active
        or not.

        Parameters
        ----------
        self : class
            A Pointer to the class itself to be able
            to access all the neccessary items set
            in the class.
        """
        data = None
        self.remark_items()
        while self.loop:
            self.stdscr.clear()
            self.stdscr.addstr(self.init_scr_y, self.init_scr_x,
                               self.menu_title,
                               curses.color_pair(1) + curses.A_BOLD)
            for k, menu_item in enumerate(self.items):
                self.stdscr.addstr(self.init_scr_y + 1 + k, self.init_scr_x,
                                   menu_item.title if menu_item.title in ("Cancel", "Done")\
                                   else str(menu_item), menu_item.get_style())
            data = self.actions()
        if data is not None:
            return data

    def actions(self):
        """
        After rendering the menu for the user, it will stop
        and start capturing keys from the user. The key stroke
        will then be used to determine what to do based on the
        different types of menu items as well as the position
        and whether or not to exit the menu.
        """
        key = self.stdscr.getkey()
        current_item = self.items[self.cursor_position]
        if current_item.title in ("Cancel", "Exit") and key == "\n"\
                or key in ("q", "Q"):
            self.exit_menu()
            return None

        if key in ("KEY_UP", "KEY_DOWN"):
            self.cursor_position = (self.cursor_position + 1) % len(self.items)\
                if key == "KEY_DOWN" else (self.cursor_position - 1) % len(self.items)\
                if key == "KEY_UP" else self.cursor_position
            self.remark_items()

        if isinstance(current_item, MenuItem) and key == "\n"\
                and not isinstance(current_item, ItemSingleSelection)\
                and not isinstance(current_item, ItemMultiSelection):
            current_item.take_action()

        if isinstance(current_item, ItemSingleSelection) and key == "\n"\
                and not isinstance(current_item, ItemMultiSelection):
            self.exit_menu()
            return current_item.title

        if isinstance(current_item, ItemMultiSelection):
            data = None
            if key == "\n" and current_item.title != "Done":
                self.items[self.cursor_position].select()
            elif key == "\n" and current_item.title == "Done":
                data = [i.title for i in self.items if i.is_selected]
                self.exit_menu()
            return data
        return None

    def exit_menu(self):
        """
        Stops the loop that draws and captures
        the user keystrokes and menu.
        """
        self.loop = False

    def remark_items(self):
        """
        Set all menu items as inactive except
        for the menu item that is on the current
        position. 
        """
        for item in self.items:
            item.set_active(self.cursor_position)



"""
TODO: Write a docstring
"""
import curses

from icecream import ic

from menu_zerobeef.data import (ItemMultiSelection, ItemSingleSelection,
                                MenuItem)
from menu_zerobeef.menu import ApplicationMenu


def multi_selection(stdscr):
    """
    This function creates and presents the user
    with an example menu where you can select
    multiple items. If the user selects "Cancel",
    nothing is returned. However, if the user
    selects "Done", then it returns whatever is
    selected, if nothing is selected then [] is
    returned. 
    """
    curses.start_color()
    curses.use_default_colors()
    items = [
        "system", "os", "windows", "linux", "mac", "apple", "pear", "dog",
        "Cancel", "Done"
    ]
    multi_select_items = [
        ItemMultiSelection(t, p) for p, t in enumerate(items)
    ]
    menu_title = "Multiple choice menu example. Which of these are an os or contains os?"
    menu = ApplicationMenu(stdscr, (multi_select_items, menu_title))
    return menu.show_menu()


def single_selection(stdscr):
    """
    This function creates and presents the user
    with an example menu where you can select
    one item out of a list of items. If the user
    selects "Cancel", then nothing will be returned.
    However, if the user selects anything else the
    title for that item will be returned and not
    the menu item instance.
    """
    curses.start_color()
    curses.use_default_colors()
    items = [
        "system", "os", "windows", "linux", "mac", "apple", "pear", "dog",
        "Cancel"
    ]
    single_select_items = [
        ItemSingleSelection(t, p) for p, t in enumerate(items)
    ]
    menu_title = "Single selection menu example. Which one is related to apple?"

    menu = ApplicationMenu(stdscr, (single_select_items, menu_title))
    return menu.show_menu()


def normal_menu(stdscr):
    """
    This function creates a menu and presents
    the user with a normal menu that has sub-
    menus. Whenever the user goes to a specific
    item that does not contain a submenu, the
    action for that item is executed. If the
    item contains a pointer to a submenu, that
    submenu will then be drawn on screen and 
    the main menu will wait in the background.
    """
    curses.start_color()
    curses.use_default_colors()
    sub_menu = [(print, "SubFirst"), (print, "SubSecond"), (None, "Exit")]
    me = [MenuItem(*t, p, None) for p, t in enumerate(sub_menu)]
    sub_title = "Submenu"
    sub_app_menu = ApplicationMenu(stdscr, (me, sub_title))
    items = [(ic, "First"), (ic, "Second"), (ic, "Third"),
             (sub_app_menu.show_menu, "SubMenu"), (None, "Exit")]
    menu = [MenuItem(*t, k, None) for k, t in enumerate(items)]
    menu_title = "Main menu"
    menu = ApplicationMenu(stdscr, (menu, menu_title))
    return menu.show_menu()



"""
The data module defines three models for items in the menu.
One of the models is for a single menu item where there
might be a submenu or not. The second item is for single
selection only where there are no submenus and multichoice
can't be done. The third class is for multichoice, where
more than one item can be selected and there are no
submenus present.

Each class inherits the menu_item class where it holds
actions and submenus, however they will use same variables
but different content for easier understanding of how the
menu works.
"""
import curses


class MenuItem():
    """
    A basic menu item for use with a normal
    menu that can or might have submenus in
    it.

    Functions
    ---------
    __init__(self, action, title, position, fargs)
    __str__(self)
    set_active(self, position)
    get_style(self)
    take_action(self)

    Parameters
    ----------
    self : Any
        An object pointing to an instance
        of this class.
    action : Func
        A pointer to a function to execute
        once the menu item is selected by 
        the user.
    title : str
        A string which should be displayed
        in the menu when drawing it on screen
    position : int
        An integer which is used to determine
        whether the menu item is active or not.
    fargs : tuple
        A tuple with extra items that should be
        sent further to the action when the item
        is selected, works almost like a pipe
    """

    def __init__(self, action, title, position, fargs):
        self.action = action
        self.title = title
        self.position = position
        self.fargs = fargs
        self.is_active = False
        curses.init_pair(10, curses.COLOR_WHITE, curses.COLOR_BLACK)
        curses.init_pair(11, curses.COLOR_BLACK, -1)

    def __str__(self):
        return str(self.title)

    def set_active(self, position):
        """
        Takes two parameters and checks whether cur_pos
        is the same as the item position that got set
        from the beginning. If this is the case it sets
        self.is_active to True otherwise False

        Parameters
        ----------
        self : object
            The pointer to the class itself

        Returns
        -------
        None
        """
        if position == self.position or\
                self.is_active:
            self.is_active = not self.is_active

    def get_style(self):
        """
        Returns the style specific to the position that
        the cursor is on.

        Parameters
        ----------
        self: object
            The pointer to the class itself

        Returns
        -------
        color_pair
            A color pair used in curses that
            will show the user what item is
            selected or not.
        """
        return curses.color_pair(10) + curses.A_BOLD if self.is_active else\
            curses.color_pair(11)

    def take_action(self):
        """
        Takes no parameter as input. Checks wether the
        item that is set to the variable action in
        the self object is callable. If the item is
        a callable then we can call it as we want.
        Otherwise we would do anything else with the
        data. Probably ignore the item which is set
        at that variable for the moment being.
        """
        if callable(self.action) and self.fargs is None:
            self.action()
        elif callable(self.action) and self.fargs is not None:
            self.action(*self.fargs)


class ItemSingleSelection(MenuItem):
    """
    TODO: Write a docstring
    """

    def __init__(self, title, position):
        super().__init__(None, title, position, None)
        self.single_choice = True

    def __str__(self):
        return f"{self.title} <-" if self.is_active else f"{self.title}"

    def get_style(self):
        return super().get_style()


class ItemMultiSelection(MenuItem):
    """
    ItemMultiSelection is used when the menu is in
    a mode where multiple items can be selected.
    The menu item inherits the same properties from
    the MenuItem class and adds some extra functionality
    or changes the functions to differentiate from
    the super class.

    Parameters
    ----------
    title : str
        An items title which should be displayed in
        the menu.
    position : int
        Where in the menu is it located at
    """

    def __init__(self, title, position):
        super().__init__(None, title, position, None)
        self.is_selected = False
        curses.init_pair(12, curses.COLOR_GREEN, curses.COLOR_BLACK)
        curses.init_pair(13, curses.COLOR_GREEN, -1)

    def __str__(self):
        return f"[x] {self.title} <-" if self.is_selected and self.is_active else\
            f"[x] {self.title}" if self.is_selected else\
            f"[ ] {self.title} <-" if self.is_active else\
            f"[ ] {self.title}"

    def select(self):
        """
        The action can be used when the menu type
        is a multiple choice menu. The function
        works in the same wat as self.set_active
        by negating is own variable. See the
        function definition for set_active
        """
        self.is_selected = not self.is_selected

    def get_style(self):
        """
        Changes how the menu item is displayed on
        screen. If the menu item is not selected
        but active or selected but not active or
        selected and active, then different styles
        are applited to the menu item.
        """
        return super().get_style() if not self.is_selected else\
            curses.color_pair(13) if self.is_selected and not self.is_active else\
            curses.color_pair(12) + curses.A_BOLD



from setuptools import find_packages, setup

setup(
    name="menu",
    version="0.2",
    author="Erik",
    author_email="zerobeef@icloud.com",
    description="A simple menu program for command line enthusiasts made possible using curses",
    url="https://github.com/erikkamph/project_menu",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    python_requires='>=3.7'
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import curses

from icecream import ic

from menu_zerobeef.example import (multi_selection, normal_menu,
                                   single_selection)

menus = {1: normal_menu, 2: single_selection, 3: multi_selection}

if __name__ == "__main__":
    print("1. Menu with submenus")
    print("2. Single selection menu")
    print("3. Multiple choice Menu")
    a = input("Which example do you want to run?")
    b = curses.wrapper(menus[int(a)])
    ic(b)
